function [decoded_values] = fread_bcd(fid, num_values, offset, scale_factor, num_digits)

% Copyright (c) 2007-2014 University of Wisconsin Regents.
%
% This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see <http://www.gnu.org/licenses/>.

% Read and decode binary coded decimal values from a file. The number of bits
% read will be 4*num_digits*num_values.
%
% Inputs:
%   fid           file identifier
%   num_values    number of values to read (>1 not tested)
%   offset        not used
%   scale_factor  coefficient from XML database, used to calculate
%                 decimal position
%   num_digits    abs val is the number of binary-coded digits;
%                 <0 indicates the first bit is a sign bit  
%
% Output
%   decoded_values  Array of decoded values
%
% FIXME: should eliminate the parameter 'offset' which is not used
% but is included to preserve backward compatibility. Should also
% review the other parameters. 

% calculate decimal position
decimal_position = log10(scale_factor) + abs(num_digits) + 1;

n = abs(num_digits);

for i=1:num_values
    
    % read BCD values into an array of 8-bit unsigned 
    arr = fread(fid, n, '*ubit4', 'ieee-be');

    % if BCD value is signed, determine sign and zero out the sign bit
    sign_factor = 1;
    if(num_digits < 0 && (bitget(arr(1), 4) == 1)) 
        sign_factor = -1;
        arr(1) = bitand(arr(1), 7);
    end
    
    % decode values
    decoded_values(i) = 0.0;
    for j=1:n
        decoded_values(i) = decoded_values(i) + (double(arr(j)) * 10^(decimal_position - j - 1));
    end
    decoded_values(i) = decoded_values(i) * sign_factor;
    
end